require "numru/gphys"

# ToDo :: data missing handling ; use scaled_put/scaled_get

module NumRu
   class GPhys
      module NetCDF_IO

	 module_function

	 def open(file, varname)
	    if file.is_a?(String)
	       file = NetCDF.open(file)
	    elsif ! file.is_a?(NetCDF)
	       raise ArgumentError, "1st arg must be a NetCDF or a file name"
	    end
	    ncvar = file.var(varname)
	    data = VArrayNetCDF.new(ncvar)
	    axlocnames = ncvar.dim_names    # NC User's Guide
	    rank = ncvar.rank
	    bare_index = [ false ] * rank # --> true if coord var is not found
	    cell = [ false ] * rank       # NC User's Guide gives no rule
            cell_center = [ true ] * rank # NC User's Guide gives no rule

            axes = Array.new
	    var_names = file.var_names
            for i in 0...rank
	       if var_names.include?(axlocnames[i])
		  axloc = VArrayNetCDF.new( file.var(axlocnames[i]) )
	       else
		  bare_index[i]=true
		  na = NArray.float(file.dim(dimnames[i]).length).indgen!
		  axloc = VArray.new( na )
	       end
	       axis = Axis.new(cell[i],bare_index[i])
	       if !cell[i]
		  axis.set_loc( axloc )
	       else
		  if cell_center[i]
		     p "Sorry: method to find bounds has not been defined"
		     axis.set_cell_guess_bounds(axloc).set_loc_to_center
		  else
		     raise "Sorry: method to find center has not been defined"
		  end
	       end

	       #p "yet-to-be-defined: method to define aux coord vars"

	       axis.set_default_algorithms

	       axes[i] = axis
	    end

	    grid = Grid.new( *axes )

	    GPhys.new(grid,data)
	 end

#	 module_function :open

      end
   end
end

######################################################
if $0 == __FILE__
   include NumRu

   file = NetCDF.open("uwnd.mon.ltm.nc")
   uwnd = GPhys::NetCDF_IO.open(file,"uwnd")
   p uwnd.name, uwnd.shape_current
   u_jan = uwnd[0..-1,0..-1,0..-1,0]
   p u_jan.name, u_jan.shape_current
   u_jan2 = uwnd[true,true,true,0..0]
   p u_jan2.name, u_jan2.shape_current

   u_jan_zonalmean = u_jan.mean(0)
   p u_jan_zonalmean.val

   uu = ( u_jan - u_jan_zonalmean )
   p uu.val[0,true,true]
end
